<?php
namespace Aelia\WC\CurrencySwitcher\Integrations\Blocks\WC;
if(!defined('ABSPATH')) { exit; } // Exit if accessed directly

use Aelia\WC\CurrencySwitcher\Definitions;
use Aelia\WC\CurrencySwitcher\Integration\Blocks\WC_Aelia_CS_Base_Block_Integration;
use Aelia\WC\CurrencySwitcher\Settings;

/**
 * Implements support for the WooCommerce Checkout Block.
 *
 * @since 4.10.0.210312
 */
class WC_Aelia_CS_WooCommerce_Checkout_Block_Integration extends WC_Aelia_CS_Base_Block_Integration {
	/**
	 * The URI used by the WC Checkout Block to update the user data on the checkout page.
	 *
	 * @var string
	 */
	const URI_CHECKOUT_ACTION_UPDATE_CUSTOMER = 'wc/store/cart/update-customer';

	/**
	 * Constructor.
	 */
	public function __construct() {
		parent::__construct();
		add_filter('woocommerce_rest_is_request_to_rest_api', [$this, 'woocommerce_rest_is_request_to_rest_api'], 1);
	}

	/**
	 * Intercepts requests to the REST API, maded to update the user data at checkout. If option "force currency
	 * by country" is enabled, this method tries to update customer's country.
	 *
	 * @param bool $is_request_to_rest_api
	 * @return bool
	 */
	public function woocommerce_rest_is_request_to_rest_api($is_request_to_rest_api) {
		if($is_request_to_rest_api && self::rest_request_matches_uri(self::URI_CHECKOUT_ACTION_UPDATE_CUSTOMER)) {
			// Check if the "force currency by country" option is enabled
			$country_source = $this->cs()->force_currency_by_country();

			if($country_source !== Settings::OPTION_DISABLED) {
				// Fetch the content from the request
				$request_body = json_decode(@file_get_contents('php://input'), true);

				$this->get_logger()->debug(__('Handling request to update user date from WooCommerce Checkout Block.', Definitions::TEXT_DOMAIN), [
					'Request URI' => $_SERVER['REQUEST_URI'],
					'Request Body' => $request_body,
				]);

				if(is_array($request_body)) {
					// Extract customer's country from the request, so that it can be set to select the currency
					if($country_source === Settings::OPTION_SHIPPING_COUNTRY) {
						$customer_country = $request_body['shipping_address']['country'] ?? ($request_body['billing_address']['country'] ?? '');
					}
					else {
						$customer_country = ($request_body['billing_address']['country'] ?? '');
					}

					$this->get_logger()->debug(__('Customer country fetched from the data passed by the WooCommerce Checkout Block.', Definitions::TEXT_DOMAIN), [
						'Country Source' => $country_source,
						'Customer Country' => $customer_country,
					]);

					// Simulate an explicit selection of customer's country. This will also set the active currency
					if(!empty($customer_country)) {
						$_POST['aelia_customer_country'] = $customer_country;
					}
				}
			}
		}

		return $is_request_to_rest_api;
	}
}